from logging.config import dictConfig

# look into: https://www.packetmischief.ca/2017/10/25/3-ways-to-fail-at-logging-with-flask/
# source: https://docs.python.org/3/howto/logging-cookbook.html (An example dictionary-based configuration)
# discussion: https://github.com/pallets/flask/issues/2023
FORMATTERS = {
    'verbose': {
        'format': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        },

    'simple': {
        'format': '%(levelname)s %(message)s'
        },
    }

HANDLERS = {
    'console': {
        'level': 'DEBUG',
        'class': 'logging.StreamHandler',
        'formatter': 'verbose'
        },
    }

LOGGERS = {
    'flask': {'handlers': ['console'], 'level': 'INFO', 'propagate': True},
    'portal': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    'tests': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    'management': {'handlers': ['console'], 'level': 'INFO', 'propagate': True},
    'app': {'handlers': ['console'], 'level': 'INFO', 'propagate': True},
    }

LOGGING_CONF = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': FORMATTERS,
    'handlers': HANDLERS,
    'loggers': LOGGERS,
    }


def load_config(conf_type=None):
    """Loads config based on the config type
    Args:
        conf_type(str): Config type available (dev, test, prod)

    """
    dictConfig(LOGGING_CONF)
