import logging
from emails import Message

from flask_emails import Message
from emails.template import JinjaTemplate as T

from portal.database.models import User

log = logging.getLogger(__name__)


class EmailMessage(object):
    def __init__(self, subject, body, **kwargs):
        self._subject = T(subject)
        self._body = T(body)
        self._params = kwargs

    @property
    def subject(self):
        return self._subject

    @property
    def body(self):
        return self._body

    def create_message(self) -> Message:
        message = Message(
            subject=self.subject,
            html=self.body,
            **self._params
            )
        return message


USERS = dict(
    created=EmailMessage(
        subject="[KONTR] User created: {{username}}",
        body="User {{username}} has been created"
        ),
    deleted=EmailMessage(
        subject="[KONTR] User deleted: {{username}}",
        body="User {{username}} has been deleted"
        ),
    password_update=EmailMessage(
        subject="[KONTR] User's password updated: {{username}}",
        body="User {{username}}'s password has been updated"
        ),
    updated=None
    )

COURSES = dict(
    created=None,
    deleted=None,
    updated=None
    )

PROJECTS = dict(
    created=None,
    deleted=None,
    updated=None
    )

NOTIFICATIONS = dict(
    new=EmailMessage(
        subject="Notification: {{subject}}",
        body="{{content}}"
        )
    )

SUBMISSIONS = dict(
    student=EmailMessage(
        subject='[KONTR] Submission evaluation for {{username}}',
        body="{{content}}"
        ),
    teacher=EmailMessage(
        subject='[KONTR] Submission evaluation for {{username}}',
        body="{{content}}"
        )
    )

MESSAGES = dict(
    users=USERS,
    courses=COURSES,
    projects=PROJECTS,
    notifications=NOTIFICATIONS,
    submissions=SUBMISSIONS,
    )


def get_email_message(collection: str, action: str) -> EmailMessage:
    return MESSAGES[collection][action]


def create_message(collection: str, action: str) -> Message:
    return get_email_message(collection=collection, action=action).create_message()


def send_message_to_user(user: User = None, message: Message = None,
                         context: dict = None):
    email = user.email
    to = (user.name, email)
    if email and message:
        log.debug(f"[EMAIL] Send message to user({user.id}[{user.username}] :: {email}): "
                  f"{message}")
        message.send(to=to, render=context)


def notify_user(user: User, collection: str, action: str, context: dict = None):

    message = create_message(collection=collection, action=action)
    send_message_to_user(user=user, message=message, context=context)
